@@ -1,21 +1,27 @@
+"""
+    TODO move permission checks to the dispatch view thingee
+"""
+
 from datetime import datetime
 
 from django.conf import settings
 from django.core.urlresolvers import reverse
+from django.forms import widgets
 from django.forms.widgets import DateTimeInput, CheckboxInput
-from django.http import HttpResponseForbidden
+from django.http import HttpResponseForbidden, HttpResponseRedirect
+from django.views.generic.edit import DeletionMixin
 from django.views.generic import DetailView
 from django.views.generic import ListView
 from django.views.generic import TemplateView
 from django.views.generic.edit import FormView
 
-
 from mongoengine.fields import EmbeddedDocumentField, ListField
 
 from mongonaut.forms import DocumentListForm
 from mongonaut.forms import DocumentDetailForm
 from mongonaut.forms import document_detail_form_factory
 from mongonaut.mixins import MongonautViewMixin
+from mongonaut.utils import is_valid_object_id
 
 class IndexView(ListView, MongonautViewMixin):
 
@@ -39,6 +45,21 @@ class DocumentListView(FormView, MongonautViewMixin):
     success_url = '/'
     template_name = "mongonaut/document_list.html"
     
+    def get_qset(self, queryset, q):
+        if self.mongoadmin.search_fields and q:
+            params = {}
+            for field in self.mongoadmin.search_fields:
+                if field == 'id':
+                    # check to make sure this is a valid ID, otherwise we just continue
+                    if is_valid_object_id(q):
+                        return queryset.filter(id=q)
+                    continue
+                search_key = "{field}__icontains".format(field=field)
+                params[search_key] = q
+
+            queryset = queryset.filter(**params)        
+        return queryset
+    
     def get_queryset(self):
         if hasattr(self, "queryset") and self.queryset:
             return self.queryset
@@ -51,12 +72,8 @@ class DocumentListView(FormView, MongonautViewMixin):
         # search. move this to get_queryset        
         # search. move this to get_queryset
         q = self.request.GET.get('q')
-        if self.mongoadmin.search_fields and q:
-            params = {}
-            for field in self.mongoadmin.search_fields:
-                search_key = "{field}__icontains".format(field=field)
-                params[search_key] = q
-            queryset = queryset.filter(**params)
+        query = self.get_qset(queryset, q)
+
         
         ### Start pagination
         ### Note: 
@@ -90,6 +107,9 @@ class DocumentListView(FormView, MongonautViewMixin):
         
     def get_context_data(self, **kwargs):
         context = super(DocumentListView, self).get_context_data(**kwargs)
+        context = self.get_permissions(context)
+        if not context['has_view_permission']:
+            return HttpResponseForbidden("You do not have permissions to view this content.")
         
         context['object_list'] = self.get_queryset()        
         context['document'] = self.document
@@ -121,7 +141,6 @@ class DocumentListView(FormView, MongonautViewMixin):
         if self.mongoadmin.search_fields:
             context['search_field'] = True
 
-        context = self.get_permissions(context)
         return context
 
     def post(self, request, *args, **kwargs):
@@ -143,7 +162,10 @@ class DocumentDetailView(TemplateView, MongonautViewMixin):
     
     def get_context_data(self, **kwargs):
         context = super(DocumentDetailView, self).get_context_data(**kwargs)
-        self.set_mongonaut_base()
+        self.set_mongoadmin()        
+        context = self.get_permissions(context)
+        if not context['has_view_permission']:
+            return HttpResponseForbidden("You do not have permissions to view this content.")
         self.document_type = getattr(self.models, self.document_name)
         self.ident = self.kwargs.get('id')
         self.document = self.document_type.objects.get(id=self.ident)
@@ -161,8 +183,6 @@ class DocumentDetailView(TemplateView, MongonautViewMixin):
                 continue
             context['keys'].append(key)
             
-        self.set_mongoadmin()
-        context = self.get_permissions(context)
 
         return context
  
@@ -175,31 +195,36 @@ class DocumentDetailEditFormView(FormView, MongonautViewMixin):
     success_url = '/'
     
     def get_success_url(self):
-        self.set_mongonaut_base()  
-        return reverse('document_detail_edit_form', kwargs={'app_label':self.app_label,'document_name':self.document_name,'id':self.kwargs.get('id')})    
+        self.set_mongonaut_base() 
+        return reverse('document_detail_edit_form', kwargs={'app_label':self.app_label,'document_name':self.document_name,'id':self.kwargs.get('id')})
     
     def get_context_data(self, **kwargs):
         context = super(DocumentDetailEditFormView, self).get_context_data(**kwargs)
-        self.set_mongonaut_base()
+        self.set_mongoadmin()        
+        context = self.get_permissions(context)
+        if not context['has_edit_permission']:
+            return HttpResponseForbidden("You do not have permissions to edit this content.")
         self.document_type = getattr(self.models, self.document_name)
         self.ident = self.kwargs.get('id')
-        self.document = self.document_type.objects.get(id=self.ident)
+        self.document = self.document_type.objects.get(id=self.ident)        
         
         context['document'] = self.document
-        context['app_label'] = self.app_label  
+        context['app_label'] = self.app_label
         context['document_name'] = self.document_name
-            
-        self.set_mongoadmin()
-        context = self.get_permissions(context)
 
         return context
         
     def get_form(self, DocumentDetailForm):
-        self.set_mongonaut_base()
+        self.set_mongoadmin()
+        context = self.get_permissions({})
+        if not context['has_edit_permission']:
+            return HttpResponseForbidden("You do not have permissions to edit this content.")
+        
         self.document_type = getattr(self.models, self.document_name)
         self.ident = self.kwargs.get('id')
         self.document = self.document_type.objects.get(id=self.ident)
-        self.form = DocumentDetailForm()        
+        self.form = DocumentDetailForm()                
+            
         self.form = document_detail_form_factory(form=self.form, document_type=self.document_type, initial=self.document)
         if self.request.method == 'POST':
             self.form.data = self.request.POST
@@ -209,7 +234,6 @@ class DocumentDetailEditFormView(FormView, MongonautViewMixin):
                     if 'readonly' in field.widget.attrs:
                         # For _id
                         # TODO - make the ones below work
-                        # for ReferenceField - like <class 'articles.models.User'> on Blog                        
                         # For ListField - like 'field': <mongoengine.fields.StringField object at 0x101b51810>,                                
                         # For EmbeddedDocumentField
                         continue       
@@ -218,20 +242,24 @@ class DocumentDetailEditFormView(FormView, MongonautViewMixin):
                         format = field.widget.format
                         setattr(self.document, key, datetime.strptime(self.request.POST[key], format))
                         continue
-                        
+
+
                     if isinstance(field.widget, CheckboxInput):
                         if key in self.request.POST:
                             setattr(self.document, key, True)
                         else:
                             setattr(self.document, key, False)
                         continue
+                        
+                    if isinstance(field.widget, widgets.Select):
+                        # TODO - support reference fields
+                        continue
 
                     # for strings
                     setattr(self.document, key, self.request.POST[key])
                 self.document.save()
                 # TODO add message for save
-
-            
+                
         return self.form
         
 class DocumentDetailAddFormView(FormView, MongonautViewMixin):
@@ -251,15 +279,14 @@ class DocumentDetailAddFormView(FormView, MongonautViewMixin):
             self.document = self.document_type.objects.get(id=self.ident)
         """
         context = super(DocumentDetailAddFormView, self).get_context_data(**kwargs)
-        self.set_mongonaut_base()
-        self.document_type = getattr(self.models, self.document_name)
-
-        context['app_label'] = self.app_label  
-        context['document_name'] = self.document_name
-
-        self.set_mongoadmin()
+        self.set_mongoadmin()        
         context = self.get_permissions(context)
-
+        if not context['has_add_permission']:
+            return HttpResponseForbidden("You do not have permissions to edit view content.")
+        self.document_type = getattr(self.models, self.document_name)
+        
+        context['app_label'] = self.app_label
+        context['document_name'] = self.document_name
         return context
 
     def get_form(self, DocumentDetailForm):
@@ -298,10 +325,28 @@ class DocumentDetailAddFormView(FormView, MongonautViewMixin):
                 self.document.save()
                 # TODO add message for save
 
+        return self.form
 
-        return self.form        
-        
+class DocumentDeleteView(TemplateView, DeletionMixin, MongonautViewMixin):
+    """ :args: <app_label> <document_name> <id> 
     
+        TODO - implement a GET view for confirmation
+    """
+    
+    success_url = "/"
+    template_name = "mongonaut/document_delete.html"
+    
+    def get_success_url(self):
+        self.set_mongonaut_base()  
+        return reverse('document_list', kwargs={'app_label':self.app_label,'document_name':self.document_name})
+
+    def get_object(self):
+        self.set_mongoadmin()                
+        self.document_type = getattr(self.models, self.document_name)
+        self.ident = self.kwargs.get('id')
+        self.document = self.document_type.objects.get(id=self.ident)
+        return self.document        
+
 
 class EmbeddedDocumentDetailView(DetailView, MongonautViewMixin):
     """ :args: <app_label> <document_name> <id> <???> """
